import { Constants, getTranslationPrefix, sanitizeTextByAvailableSanitizer, } from '@slickgrid-universal/common';
import { BindingHelper } from '@slickgrid-universal/binding';
export class SlickPaginationComponent {
    constructor(paginationService, pubSubService, sharedService, translaterService) {
        var _a, _b, _c, _d, _e, _f, _g;
        this.paginationService = paginationService;
        this.pubSubService = pubSubService;
        this.sharedService = sharedService;
        this.translaterService = translaterService;
        this._enableTranslate = false;
        this._subscriptions = [];
        this.firstButtonClasses = '';
        this.lastButtonClasses = '';
        this.prevButtonClasses = '';
        this.nextButtonClasses = '';
        // text translations (handled by i18n or by custom locale)
        this.textItemsPerPage = 'items per page';
        this.textItems = 'items';
        this.textOf = 'of';
        this.textPage = 'Page';
        this._bindingHelper = new BindingHelper();
        this._bindingHelper.querySelectorPrefix = `.${this.gridUid} `;
        this.currentPagination = this.paginationService.getFullPagination();
        this._enableTranslate = (_b = (_a = this.gridOptions) === null || _a === void 0 ? void 0 : _a.enableTranslate) !== null && _b !== void 0 ? _b : false;
        this._locales = (_d = (_c = this.gridOptions) === null || _c === void 0 ? void 0 : _c.locales) !== null && _d !== void 0 ? _d : Constants.locales;
        if (this._enableTranslate && (!this.translaterService || !this.translaterService.translate)) {
            throw new Error('[Slickgrid-Universal] requires a Translate Service to be installed and configured when the grid option "enableTranslate" is enabled.');
        }
        this.translatePaginationTexts(this._locales);
        if (this._enableTranslate && ((_e = this.pubSubService) === null || _e === void 0 ? void 0 : _e.subscribe)) {
            const translateEventName = (_g = (_f = this.translaterService) === null || _f === void 0 ? void 0 : _f.eventName) !== null && _g !== void 0 ? _g : 'onLanguageChange';
            this._subscriptions.push(this.pubSubService.subscribe(translateEventName, () => this.translatePaginationTexts(this._locales)));
        }
        // Anytime the pagination is initialized or has changes,
        // we'll copy the data into a local object so that we can add binding to this local object
        this._subscriptions.push(this.pubSubService.subscribe('onPaginationRefreshed', (paginationChanges) => {
            for (const key of Object.keys(paginationChanges)) {
                this.currentPagination[key] = paginationChanges[key];
            }
            this.updatePageButtonsUsability();
            const pageFromToElm = document.querySelector(`.${this.gridUid} span.page-info-from-to`);
            if (pageFromToElm === null || pageFromToElm === void 0 ? void 0 : pageFromToElm.style) {
                pageFromToElm.style.display = (this.currentPagination.totalItems === 0) ? 'none' : '';
            }
        }));
    }
    get availablePageSizes() {
        return this.paginationService.availablePageSizes || [];
    }
    get dataFrom() {
        return this.paginationService.dataFrom;
    }
    get dataTo() {
        return this.paginationService.dataTo;
    }
    get itemsPerPage() {
        return this.paginationService.itemsPerPage;
    }
    set itemsPerPage(count) {
        this.paginationService.changeItemPerPage(count);
    }
    get pageCount() {
        return this.paginationService.pageCount;
    }
    get pageNumber() {
        return this.paginationService.pageNumber;
    }
    set pageNumber(_page) {
        // the setter has to be declared but we won't use it, instead we will use the "changeToCurrentPage()" to only update the value after ENTER keydown event
    }
    get grid() {
        return this.sharedService.slickGrid;
    }
    get gridOptions() {
        return this.sharedService.gridOptions;
    }
    get gridUid() {
        var _a, _b;
        return (_b = (_a = this.grid) === null || _a === void 0 ? void 0 : _a.getUID()) !== null && _b !== void 0 ? _b : '';
    }
    get totalItems() {
        return this.paginationService.totalItems;
    }
    get isLeftPaginationDisabled() {
        return this.pageNumber === 1 || this.totalItems === 0;
    }
    get isRightPaginationDisabled() {
        return this.pageNumber === this.pageCount || this.totalItems === 0;
    }
    dispose() {
        // also dispose of all Subscriptions
        this.pubSubService.unsubscribeAll(this._subscriptions);
        this._bindingHelper.dispose();
        this._paginationElement.remove();
    }
    renderPagination(gridParentContainerElm) {
        const paginationTemplate = require('./slick-pagination.component.html');
        if (paginationTemplate) {
            const temp = document.createElement('div');
            temp.innerHTML = sanitizeTextByAvailableSanitizer(this.gridOptions, paginationTemplate, { ALLOW_DATA_ATTR: true });
            this._paginationElement = temp.firstChild;
            this._paginationElement.classList.add(this.gridUid, 'pager');
            this._paginationElement.style.width = '100%';
            if ((gridParentContainerElm === null || gridParentContainerElm === void 0 ? void 0 : gridParentContainerElm.appendChild) && this._paginationElement) {
                gridParentContainerElm.appendChild(this._paginationElement);
                this.renderPageSizes();
                this.addBindings();
                this.addEventListeners();
                this.updatePageButtonsUsability();
            }
        }
    }
    /** Render and fill the Page Sizes <select> element */
    renderPageSizes() {
        const selectElm = document.querySelector(`.${this.gridUid} .items-per-page`);
        if (selectElm && Array.isArray(this.availablePageSizes)) {
            for (const option of this.availablePageSizes) {
                const opt = document.createElement('option');
                opt.value = `${option}`;
                opt.text = `${option}`;
                selectElm.appendChild(opt);
            }
        }
    }
    /** Add some DOM Element bindings */
    addBindings() {
        this._bindingHelper.addElementBinding(this, 'firstButtonClasses', 'li.page-item.seek-first', 'className');
        this._bindingHelper.addElementBinding(this, 'prevButtonClasses', 'li.page-item.seek-prev', 'className');
        this._bindingHelper.addElementBinding(this, 'lastButtonClasses', 'li.page-item.seek-end', 'className');
        this._bindingHelper.addElementBinding(this, 'nextButtonClasses', 'li.page-item.seek-next', 'className');
        this._bindingHelper.addElementBinding(this.currentPagination, 'dataFrom', 'span.item-from', 'textContent');
        this._bindingHelper.addElementBinding(this.currentPagination, 'dataTo', 'span.item-to', 'textContent');
        this._bindingHelper.addElementBinding(this.currentPagination, 'totalItems', 'span.total-items', 'textContent');
        this._bindingHelper.addElementBinding(this.currentPagination, 'pageCount', 'span.page-count', 'textContent');
        this._bindingHelper.addElementBinding(this.currentPagination, 'pageNumber', 'input.page-number', 'value', 'change', this.changeToCurrentPage.bind(this));
        this._bindingHelper.addElementBinding(this.currentPagination, 'pageSize', 'select.items-per-page', 'value');
        // locale text changes
        this._bindingHelper.addElementBinding(this, 'textItems', 'span.text-items', 'textContent');
        this._bindingHelper.addElementBinding(this, 'textItemsPerPage', 'span.text-item-per-page', 'textContent');
        this._bindingHelper.addElementBinding(this, 'textOf', 'span.text-of', 'textContent');
        this._bindingHelper.addElementBinding(this, 'textPage', 'span.text-page', 'textContent');
    }
    /** Add some DOM Element event listeners */
    addEventListeners() {
        this._bindingHelper.bindEventHandler('.icon-seek-first', 'click', this.changeToFirstPage.bind(this));
        this._bindingHelper.bindEventHandler('.icon-seek-end', 'click', this.changeToLastPage.bind(this));
        this._bindingHelper.bindEventHandler('.icon-seek-next', 'click', this.changeToNextPage.bind(this));
        this._bindingHelper.bindEventHandler('.icon-seek-prev', 'click', this.changeToPreviousPage.bind(this));
        this._bindingHelper.bindEventHandler('select.items-per-page', 'change', (event) => { var _a, _b; return this.itemsPerPage = +((_b = (_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : 0); });
    }
    changeToFirstPage(event) {
        if (!this.isLeftPaginationDisabled) {
            this.paginationService.goToFirstPage(event);
        }
    }
    changeToLastPage(event) {
        if (!this.isRightPaginationDisabled) {
            this.paginationService.goToLastPage(event);
        }
    }
    changeToNextPage(event) {
        if (!this.isRightPaginationDisabled) {
            this.paginationService.goToNextPage(event);
        }
    }
    changeToPreviousPage(event) {
        if (!this.isLeftPaginationDisabled) {
            this.paginationService.goToPreviousPage(event);
        }
    }
    changeToCurrentPage(pageNumber) {
        this.paginationService.goToPageNumber(+pageNumber);
    }
    // --
    // protected functions
    // --------------------
    updatePageButtonsUsability() {
        this.firstButtonClasses = this.isLeftPaginationDisabled ? 'page-item seek-first disabled' : 'page-item seek-first';
        this.prevButtonClasses = this.isLeftPaginationDisabled ? 'page-item seek-prev disabled' : 'page-item seek-prev';
        this.lastButtonClasses = this.isRightPaginationDisabled ? 'page-item seek-end disabled' : 'page-item seek-end';
        this.nextButtonClasses = this.isRightPaginationDisabled ? 'page-item seek-next disabled' : 'page-item seek-next';
    }
    /** Translate all the texts shown in the UI, use ngx-translate service when available or custom locales when service is null */
    translatePaginationTexts(locales) {
        var _a;
        if (this._enableTranslate && ((_a = this.translaterService) === null || _a === void 0 ? void 0 : _a.translate)) {
            const translationPrefix = getTranslationPrefix(this.gridOptions);
            this.textItemsPerPage = this.translaterService.translate(`${translationPrefix}ITEMS_PER_PAGE`);
            this.textItems = this.translaterService.translate(`${translationPrefix}ITEMS`);
            this.textOf = this.translaterService.translate(`${translationPrefix}OF`);
            this.textPage = this.translaterService.translate(`${translationPrefix}PAGE`);
        }
        else if (locales) {
            this.textItemsPerPage = locales.TEXT_ITEMS_PER_PAGE || 'TEXT_ITEMS_PER_PAGE';
            this.textItems = locales.TEXT_ITEMS || 'TEXT_ITEMS';
            this.textOf = locales.TEXT_OF || 'TEXT_OF';
            this.textPage = locales.TEXT_PAGE || 'TEXT_PAGE';
        }
    }
}
//# sourceMappingURL=slick-pagination.component.js.map